/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;

/**
   This program creates an animation by repeatedly redrawing to the Java graphics window.

   This program sleeps for a short time between animation frames.
*/
public class Animation
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render
      final Scene scene = new Scene();

      // Create several Model objects.
      final Model m1 = new ObjSimpleModel(new File("../assets/apple.obj"));
      final Model m2 = new ObjSimpleModel(new File("../assets/cow.obj"));
      final Model m3 = new ObjSimpleModel(new File("../assets/cessna.obj"));
      final Model m4 = new ObjSimpleModel(new File("../assets/galleon.obj"));
      final Model m5 = new ObjSimpleModel(new File("../assets/teapot.obj"));
      final Model m6 = new ObjSimpleModel(new File("../assets/small_rhombicosidodecahedron.obj"));
      final Model m7 = new ObjSimpleModel(new File("../assets/great_rhombicosidodecahedron.obj"));

      // Add Models to the Scene.
      scene.addModel(m1, m3, m4, m2);//apple, cessna, galleon, cow

      // Push the models away from where the camera is.
      for (final Model m : scene.modelList)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 3));
         }
      }

      Rasterize_Clip.doClipping = false; // turn off clipping


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      final FrameBufferPanel fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setResizable(false);
      jf.setVisible(true);

      // Create an ActionListener event handler object that will respond
      // to Timer events and redraw the scene into the FrameBufferPanel.
      ActionListener actionListener = new ActionListener()
      {
         // Implement the ActionListener interface.
         @Override public void actionPerformed(ActionEvent e)
         {
            // update the scene
            Model m = scene.modelList.get(0);
            for (int i = 0; i < m.vertexList.size(); ++i)
            {
               final Vertex v = m.vertexList.get(i);
               m.vertexList.set(i, new Vertex(v.x + 0.3,   // move right
                                              v.y + 0.1,   // move up
                                              v.z - 0.1)); // move back
            }
            m = scene.modelList.get(1);
            for (int i = 0; i < m.vertexList.size(); ++i)
            {
               final Vertex v = m.vertexList.get(i);
               m.vertexList.set(i, new Vertex(v.x - 0.3,   // move left
                                              v.y - 0.1,   // move down
                                              v.z - 0.1)); // move back
            }
            m = scene.modelList.get(2);
            for (int i = 0; i < m.vertexList.size(); ++i)
            {
               final Vertex v = m.vertexList.get(i);
               m.vertexList.set(i, new Vertex(v.x + 0.01, // move right
                                              v.y,
                                              v.z));
            }
            // Render again.
            final FrameBuffer fb = fbp.getFrameBuffer();
            fb.clearFB(Color.darkGray);
            Pipeline.render(scene, fb);
            fbp.update();
         }
      };

      // Create a Timer object that will generate ActionEvent
      // objects for the ActionListener object to respond to.
      final int fps = 30;
      final Timer timer = new Timer(1000/fps, actionListener);
      timer.start();
   }
}
