/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;

/**

*/
public class InteractiveHorizonDemo_R2
{
   private int n = 9;

   /**
      This constructor first instantiates the Scene object
      and initializes it with appropriate geometry. Then
      this constructor instantiates the GUI and its event
      handlers.
   */
   public InteractiveHorizonDemo_R2()
   {
      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Create the Model object.
      final Model model = new PrivateModel(n);

      // Add the Model to the Scene.
      scene.addModel(model);

      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      final FrameBufferPanel fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      Rasterize_Clip.doClipping = true;

      // Create a KeyListener event handler object that will respond to
      // keyboard events and redraw the Scene into the FrameBufferPanel.
      jf.addKeyListener( new KeyListener()
      {
         // Implement the KeyListener interface.
         @Override public void keyPressed(KeyEvent e){}
         @Override public void keyReleased(KeyEvent e){}
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );

            final char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            else if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
            }
            else if ('c' == c)
            {
               Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
               System.out.print("Clipping is turned ");
               System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
            }
            else if ('p' == c)
            {
               scene.camera.perspective = ! scene.camera.perspective;
               final String p = scene.camera.perspective ? "perspective" : "orthographic";
               System.out.println("Using " + p + " projection");
            }
            else if (('n' == c) || ('N' == c))
            {
               if ('n' == c)
                  n--;
               else
                  n++;
               if (0 == n) n = 1;
               scene.setModel(0,  new PrivateModel(n));
               System.out.println("Using n = " + n + " lines");
            }
            else if ('x' == c)
            {
               ((PrivateModel)scene.getModel(0)).moveX(-0.1);
            }
            else if ('X' == c)
            {
               ((PrivateModel)scene.getModel(0)).moveX(0.1);
            }
            else if ('y' == c)
            {
               ((PrivateModel)scene.getModel(0)).moveY(-0.1);
            }
            else if ('Y' == c)
            {
               ((PrivateModel)scene.getModel(0)).moveY(0.1);
            }
            else if ('z' == c)
            {
               ((PrivateModel)scene.getModel(0)).moveZ(-0.1);
            }
            else if ('Z' == c)
            {
               ((PrivateModel)scene.getModel(0)).moveZ(0.1);
            }

            // Render again.
            final FrameBuffer fb = fbp.getFrameBuffer();
            fb.clearFB();
            Pipeline.render(scene, fb);
            fbp.update();
         }
      });

      // Create a ComponentListener event handler object that will respond
      // to windows events and redraw the scene into the FrameBufferPanel.
      jf.addComponentListener( new ComponentListener()
      {
         // Implement the ComponentListener interface.
         @Override public void componentMoved(ComponentEvent e){}
         @Override public void componentHidden(ComponentEvent e){}
         @Override public void componentShown(ComponentEvent e){}
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );

            System.out.printf("JFrame [w = %d, h = %d]: " +
                              "FrameBufferPanel [w = %d, h = %d].\n",
                              jf.getWidth(), jf.getHeight(),
                              fbp.getWidth(), fbp.getHeight());

            // Get the new size of the FrameBufferPanel.
            final int w = fbp.getWidth();
            final int h = fbp.getHeight();

            // Create a new FrameBuffer that fits the FrameBufferPanel.
            final FrameBuffer fb = new FrameBuffer(w, h);
            fbp.setFrameBuffer(fb);
            Pipeline.render(scene, fb);
            fbp.update();
         }
      });
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveHorizonDemo_R2()
      );
   }


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, keys to move the horizon point along the x and y axes.");
      System.out.println("Use the z/Z, keys to change the length of the lines.");
      System.out.println("Use the n/N, keys to change the number of lines.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}//InteractiveHorizonDemo_R2


class PrivateModel extends Model
{
   //List<LineSgement> parallelLines;
   //List<LineSegment> backFace;

   PrivateModel(final int n)
   {
      super();

      // Add geometry to the Model.
      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < n; j++)
         {
            addVertex(new Vertex(-1.0 + (2.0*i)/(n-1), -1.0 + (2.0*j)/(n-1), -1 ),
                      new Vertex(-1.0 + (2.0*i)/(n-1), -1.0 + (2.0*j)/(n-1), -10));
            addLineSegment(new LineSegment(2*((i * n) + j),
                                           2*((i * n) + j) + 1 ));
         }
      }
   }

   public void moveX(final double deltaX)
   {
      for (final LineSegment ls : lineSegmentList)
      {
         final int farEnd = ls.index[1];
         final Vertex v1 = vertexList.get(farEnd);
         final Vertex v2 = new Vertex(v1.x + deltaX, v1.y, v1.z);
         vertexList.set(farEnd, v2);
      }
   }

   public void moveY(final double deltaY)
   {
      for (final LineSegment ls : lineSegmentList)
      {
         final int farEnd = ls.index[1];
         final Vertex v1 = vertexList.get(farEnd);
         final Vertex v2 = new Vertex(v1.x, v1.y + deltaY, v1.z);
         vertexList.set(farEnd, v2);
      }
   }

   public void moveZ(final double deltaZ)
   {
      for (final LineSegment ls : lineSegmentList)
      {
         final int farEnd = ls.index[1];
         final Vertex v1 = vertexList.get(farEnd);
         final Vertex v2 = new Vertex(v1.x, v1.y, v1.z + deltaZ);
         vertexList.set(farEnd, v2);
      }
   }
}
