/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.io.File;
import java.util.List;
import java.util.ArrayList;

/**

*/
public class InteractiveModelsAll_R2
{
   final private List<Model> modelArray = new ArrayList<>();
   private int currentModel = 0;
   private double deltaX = 0.0;
   private double deltaY = 0.0;
   private double deltaZ = 0.0;
   private double deltaS = 1.0;
   private boolean displayTransformations = false;

   /**
      This constructor first instantiates the Scene object
      and initializes it with appropriate geometry. Then
      this constructor instantiates the GUI and its event
      handlers.
   */
   public InteractiveModelsAll_R2()
   {
      // Create the Scene object that we shall render
      final Scene scene = new Scene();
/*
      // Create several Model objects, using default constructors.
      modelArray.add(new Square());
      modelArray.add(new SquareGrid());
      modelArray.add(new Circle());
      modelArray.add(new Disk());
      modelArray.add(new DiskSector());
      modelArray.add(new Ring());
      modelArray.add(new RingSector());
      modelArray.add(new Box());
      modelArray.add(new Cube());
      modelArray.add(new Cube2());
      modelArray.add(new Cube3());
      modelArray.add(new Cube4());
      modelArray.add(new Tetrahedron());
      modelArray.add(new Octahedron());
      modelArray.add(new Icosahedron());
      modelArray.add(new Dodecahedron());
      modelArray.add(new Pyramid());
      modelArray.add(new PyramidFrustum());
      modelArray.add(new TriangularPyramid());
      modelArray.add(new TriangularPrism());
      modelArray.add(new ViewFrustum());
      modelArray.add(new Cone());
      modelArray.add(new ConeSector());
      modelArray.add(new ConeFrustum());
      modelArray.add(new Cylinder());
      modelArray.add(new CylinderSector());
      modelArray.add(new Sphere());
      modelArray.add(new SphereSector());
      modelArray.add(new SphereSubdivided());
      modelArray.add(new Torus());
      modelArray.add(new TorusSector());
      modelArray.add(new ParametricCurve());
      modelArray.add(new ParametricSurface());
      modelArray.add(new SurfaceOfRevolution());
      modelArray.add(new Axes3D());
      modelArray.add(new Axes2D());
      modelArray.add(new PanelXY());
*/
      // Instantiate at least one of every Model class.
      // 2D models
      modelArray.add(new Square(1.0));
      modelArray.add(new SquareGrid(1.0, 11, 15));
      modelArray.add(new Circle(1.0, 16));
      modelArray.add(new Disk(1.0, 4, 16));
      modelArray.add(new DiskSector(1.0, Math.PI/2, 3*Math.PI/2, 4, 8));
      modelArray.add(new Ring(1.0, 0.25, 3, 16));
      modelArray.add(new RingSector(1.0, 0.25, Math.PI/2, 3*Math.PI/2, 3, 8));
      modelArray.add(new Triangle(45, 4)); // try 5 or 6
      // cubes
      modelArray.add(new Box(1.0, 1.0, 1.0));
      modelArray.add(new Cube( ));
      modelArray.add(new Cube2(4, 5, 6));
      modelArray.add(new Cube3(12, 14, 15));
      modelArray.add(new Cube4(12, 14, 15));
      // polyhedra
      modelArray.add(new Tetrahedron());
      modelArray.add(new Tetrahedron(true));
      modelArray.add(new Octahedron());
      modelArray.add(new Icosahedron());
      modelArray.add(new Dodecahedron());
      // pyramids
      modelArray.add(new Pyramid(2.0, 1.0, 5, 6));
      modelArray.add(new Pyramid(2.0, 1.0, 5, 6, true));
      modelArray.add(new PyramidFrustum(2.0, 1.0, 0.5, 2, 5));
      modelArray.add(new PyramidFrustum(1.0, 2.0, 0.5, 2, 5));
      modelArray.add(new TriangularPyramid(Math.sqrt(3)/Math.sqrt(2)));
      modelArray.add(new TriangularPyramid(1.0, 1.0, 7, 7));
      modelArray.add(new TriangularPrism(0.6, 0.5, 0.5, 3, true));
      modelArray.add(new ViewFrustumModel());
      // cones
      modelArray.add(new Cone(1.0, 1.0, 10, 16));
      modelArray.add(new ConeSector(1.0, 1.0, 0.5, 0, 2*Math.PI, 5, 16));
      modelArray.add(new ConeSector(1.0, 1.0, 0.5, Math.PI/2, 3*Math.PI/2, 5, 8));
      modelArray.add(new ConeFrustum(1.0, 0.5, 0.5, 6, 16));
      modelArray.add(new ConeFrustum(0.5, 1.0, 0.5, 6, 16));
      // cylinders
      modelArray.add(new Cylinder(0.5, 1.0, 11, 12));
      modelArray.add(new CylinderSector(0.5, 1.0, Math.PI/2, 3*Math.PI/2, 11, 6));
      // spheres
      modelArray.add(new Sphere(1.0, 15, 12));
      modelArray.add(new SphereSector(1.0, Math.PI/2, 3*Math.PI/2,
                                           Math.PI/4, 3*Math.PI/4, 7, 6));
      modelArray.add(new SphereSubdivided(5));
      modelArray.add(new SphereSubdivided(6, true, false));
      modelArray.add(new SphereSubdivided(7, false, true));
      // torus
      modelArray.add(new Torus(0.75, 0.25, 12, 16));
      modelArray.add(new TorusSector(0.75, 0.25, Math.PI/2, 3*Math.PI/2, 12, 8));
      modelArray.add(new TorusSector(0.75, 0.25, 0, 2*Math.PI,
                                                 Math.PI, 2*Math.PI, 6, 16));
      modelArray.add(new TorusSector(0.75, 0.25, 0, 2*Math.PI,
                                                -Math.PI/2, Math.PI/2, 6, 16));
      modelArray.add(new TorusSector(0.75, 0.25, Math.PI/2, 3*Math.PI/2,
                                                -Math.PI/2, Math.PI/2, 6, 8));
      // model files
      modelArray.add(new GRSModel(new File("../assets/grs/bronto.grs")));
      modelArray.add(new ObjSimpleModel(new File("../assets/cow.obj")));
      // parametric curves and surfaces
      modelArray.add(new ParametricCurve());
      modelArray.add(new ParametricSurface());
      modelArray.add(new ParametricSurface((s,t)->s*Math.cos(t*Math.PI),
                                           (s,t)->t,
                                           (s,t)->s*Math.sin(t*Math.PI),
                                           -1, 1, -1, 1, 49, 49));
      modelArray.add(new ParametricSurface(
                (u,v)->0.3*(1-u)*(3+Math.cos(v))*Math.sin(4*Math.PI*u),
                (u,v)->0.3*(3*u+(1-u)*Math.sin(v)),
                (u,v)->0.3*(1-u)*(3+Math.cos(v))*Math.cos(4*Math.PI*u),
                0, 1, 0, 2*Math.PI, 49, 49));
      modelArray.add(new SurfaceOfRevolution());
      modelArray.add(new SurfaceOfRevolution(t->0.5*(1+t*t), -1, 1, 30, 30));
      modelArray.add(new SurfaceOfRevolution(t->t, t->4*t*(1-t), 0, 1, 30, 30));
      // coordinate axes
      modelArray.add(new Axes3D(1, 1, 1));
      modelArray.add(new Axes2D(-1, 1, -1, 1, 8, 8));
      modelArray.add(new PanelXY(-4, 4, -5, 5, -5));
      modelArray.add(new PanelXZ(-1, 1, -6, 1, -0.5));
      modelArray.add(new PanelYZ(-1, 1, -6, 1, -0.5));

      // Add a model to the Scene.
      scene.addModel(modelArray.get(currentModel));

      // Push the models away from where the camera is.
      for (final Model m : modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 2));
         }
      }


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      final FrameBufferPanel fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create a KeyListener event handler object that will respond to
      // keyboard events and redraw the Scene into the FrameBufferPanel.
      jf.addKeyListener( new KeyListener()
      {
         // Implement the KeyListener interface.
         @Override public void keyPressed(KeyEvent e){}
         @Override public void keyReleased(KeyEvent e){}
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );

            final char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            else if ('d' == c)
            {
               modelArray.get(currentModel).debug = ! modelArray.get(currentModel).debug;
             //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
            }
            else if ('c' == c)
            {
               Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
               System.out.print("Clipping is turned ");
               System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
            }
            else if ('/' == c)
            {
               currentModel = (currentModel + 1) % modelArray.size();
               scene.setModel( 0, modelArray.get(currentModel) );
            }
            else if ('?' == c)
            {
               currentModel = (currentModel - 1);
               if (currentModel < 0) currentModel = modelArray.size() - 1;
               scene.setModel( 0, modelArray.get(currentModel) );
            }
            else if ('p' == c)
            {
               scene.camera.perspective = ! scene.camera.perspective;
               String p = scene.camera.perspective ? "perspective" : "orthographic";
               System.out.println("Using " + p + " projection");
            }
            else if ('m' == c) //display transformation information
            {
               displayTransformations = ! displayTransformations;
            }
            else if ('s' == c) // Scale all the models 10% smaller.
            {
               // This does NOT work with perspective projection.
               scaleModels(1.0/1.1);
            }
            else if ('S' == c) // Scale ALL the models 10% larger.
            {
               // This does NOT work with perspective projection.
               scaleModels(1.1);
            }
            else if ('x' == c) // Translate ALL the models.
            {
               moveModels(-0.1, 0, 0); // left
            }
            else if ('X' == c)
            {
               moveModels(+0.1, 0, 0); // right
            }
            else if ('y' == c)
            {
               moveModels(0, -0.1, 0); // down
            }
            else if ('Y' == c)
            {
               moveModels(0, +0.1, 0); // up
            }
            else if ('z' == c)
            {
               moveModels(0, 0, -0.1); // back
            }
            else if ('Z' == c)
            {
               moveModels(0, 0, +0.1); // forward
            }

            if (displayTransformations && ('m'==c
                 ||'s'==c||'x'==c||'y'==c||'z'==c
                 ||'S'==c||'X'==c||'Y'==c||'Z'==c))
            {
               System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                                 "deltaZ = %.2f, deltaS = %.2f\n",
                                  deltaX,        deltaY,
                                  deltaZ,        deltaS);
            }

            // Render again.
            final FrameBuffer fb = fbp.getFrameBuffer();
            fb.clearFB();
            Pipeline.render(scene, fb);
            fbp.update();
         }


         private void scaleModels(double dS)
         {
            deltaS *= dS;
            for (final Model m : modelArray)
            {
               for (int i = 0; i < m.vertexList.size(); ++i)
               {  // This does NOT work with perspective projection.
                  final Vertex v = m.vertexList.get(i);
                  m.vertexList.set(i, new Vertex(v.x * dS,
                                                 v.y * dS,
                                                 v.z * dS));
               }
            }
         }


         private void moveModels(double dX, double dY, double dZ)
         {
            deltaX += dX;
            deltaY += dY;
            deltaZ += dZ;
            for (final Model m : modelArray)
            {
               for (int i = 0; i < m.vertexList.size(); ++i)
               {
                  final Vertex v = m.vertexList.get(i);
                  m.vertexList.set(i, new Vertex(v.x + dX,
                                                 v.y + dY,
                                                 v.z + dZ));
               }
            }
         }
      });

      // Create a ComponentListener event handler object that will respond
      // to windows events and redraw the scene into the FrameBufferPanel.
      jf.addComponentListener( new ComponentListener()
      {
         // Implement the ComponentListener interface.
         @Override public void componentMoved(ComponentEvent e){}
         @Override public void componentHidden(ComponentEvent e){}
         @Override public void componentShown(ComponentEvent e){}
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );

            System.out.printf("JFrame [w = %d, h = %d]: " +
                              "FrameBufferPanel [w = %d, h = %d].\n",
                              jf.getWidth(), jf.getHeight(),
                              fbp.getWidth(), fbp.getHeight());

            // Get the new size of the FrameBufferPanel.
            final int w = fbp.getWidth();
            final int h = fbp.getHeight();

            // Create a new FrameBuffer that fits the FrameBufferPanel.
            final FrameBuffer fb = new FrameBuffer(w, h);
            fbp.setFrameBuffer(fb);
            Pipeline.render(scene, fb);
            fbp.update();
         }
      });
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveModelsAll_R2()
      );
   }


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off for the current model.");
      System.out.println("Use the '/' key to cycle through the models.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the models along the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the models.");
      System.out.println("Use the 'm' key to toggle the display of transformation information.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
