/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.framebuffer.FrameBufferPanel;

import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.io.File;

/**

*/
public class InteractiveModels_R2
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveModels_R2()
   {
      // Create an object that can hold all the client state information.
      final ClientState state = new ClientState();

      // Create the Scene object that we shall render
      state.scene = new Scene();

      // Create several Model objects.
      state.modelArray.add( new ObjSimpleModel(new File("../assets/apple.obj")) );
      state.modelArray.add( new ObjSimpleModel(new File("../assets/cow.obj")) );
      state.modelArray.add( new ObjSimpleModel(new File("../assets/galleon.obj")) );
      state.modelArray.add( new ObjSimpleModel(new File("../assets/teapot.obj")) );
      state.modelArray.add( new ObjSimpleModel(new File("../assets/cessna.obj")) );
      state.modelArray.add( new Sphere(1.0, 30, 30) );
      state.modelArray.add( new Cylinder(0.5, 1.0, 20, 20) );
      state.modelArray.add( new Torus(0.75, 0.25, 25, 25) );
      state.modelArray.add( new Cube2(15, 15, 15) );
      state.modelArray.add( new ObjSimpleModel(new File("../assets/small_rhombicosidodecahedron.obj")) );
      final Model floor = new PanelXZ(-2, 2, -2, 0);  // floor
      final Model wall  = new PanelXY(-4, 4, -1, 3);  // wall
      final Model airplane = new ObjSimpleModel(new File("../assets/cessna.obj"));

      // Add four models to the Scene.
      state.scene.addModel(state.modelArray.get(1), // cow
                           floor,
                           wall,
                           airplane);
      state.currentModel = 1;

      state.deltaX = new double[state.modelArray.size()];
      state.deltaY = new double[state.modelArray.size()];
      state.deltaZ = new double[state.modelArray.size()];
      state.deltaS = new double[state.modelArray.size()];
      java.util.Arrays.fill(state.deltaS, 1.0);

      // Position the wall, floor and airplane.
      for (int i = 0; i < wall.vertexList.size(); ++i)
      {
         final Vertex v = wall.vertexList.get(i);
         wall.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 4));
      }
      for (int i = 0; i < floor.vertexList.size(); ++i)
      {
         final Vertex v = floor.vertexList.get(i);
         floor.vertexList.set(i, new Vertex(v.x,
                                            v.y - 1,
                                            v.z - 2));
      }
      for (int i = 0; i < airplane.vertexList.size(); ++i)
      {
         final Vertex v = airplane.vertexList.get(i);
         airplane.vertexList.set(i, new Vertex(v.x + 2.5,
                                               v.y,
                                               v.z - 2));
      }
      // Push the models away from where the camera is.
      for (final Model m : state.modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 2));
         }
      }



      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      state.fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(state.fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create event handler objects for events from the JFrame.
      jf.addKeyListener(new KeyHandler(state));
      jf.addComponentListener(new ComponentHandler(state));
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      ClientState.print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveModels_R2()
      );
   }
}
