/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import javax.swing.JFrame;
import java.awt.BorderLayout;

/**
<pre>{@code
                        |
                      2 + v[1]
                        |
                        |
                        |
                        |
                        |
                  |     |
                2 +     |
                  |     |
                  |     |
                  |   1 +
            y     |     |
            |     |     |        -z
          2 +     |     |       /
            |     |     |      /
            |     |     |     /
            |     |     |    /       This 3D space is called "camera space".
            |   1 +     |   /
            |     |     |  /
            |     |     | /
            |     |     |/
            |     |  -2 +-----------------+-----------------+---- y = -2 plane
            |     |    /v[2]              1                 v[0]
          1 +     |   /
            |     |  /
            |     | /
            |     |/
            |  -1 +-----------------+----- image plane (y = -1)
            |    /                  1
            |   /
            |  /
            | /
            |/
     Camera +-----------------+---------> x-axis
           /0                 1
          /
         z-axis
}</pre>
   Render a wireframe triangle. This is just about the
   simplest possible model. It is useful for debugging.
*/
public class InteractiveTriangle_R2
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveTriangle_R2()
   {
      // Create an object that can hold all the client state information.
      final ClientState state = new ClientState();

      // Create the Scene object that we shall render.
      state.scene = new Scene();

      // Create a Model object to hold the geometry.
      final Model model = new Model("triangle");
      state.modelArray.add( model );

      // Add the Model to the Scene.
      state.scene.addModel(state.modelArray.get(0));

      // Create the geometry for the Model.
      // Vertices.
      model.addVertex(new Vertex(2.0, 0.0, -2.0),
                      new Vertex(0.0, 2.0, -2.0),
                      new Vertex(0.0, 0.0, -2.0));
      // Lines segments.
      model.addLineSegment(new LineSegment(0, 1),
                           new LineSegment(1, 2),
                           new LineSegment(2, 0));

      Rasterize_Clip.doClipping = false; // turn off clipping
      System.out.println(model);


      // Define initial dimensions for a FrameBuffer.
      final int width  = 512;
      final int height = 512;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      state.fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(state.fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create event handler objects for events from the JFrame.
      jf.addKeyListener(new KeyHandler(state));
      jf.addComponentListener(new ComponentHandler(state));
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      ClientState.print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveTriangle_R2()
      );
   }
}
