/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

/**
   This keyboard handler translates just the current model.
*/
class KeyHandler extends KeyAdapter
{
   private final ClientState client;

   public KeyHandler(ClientState client)
   {
      this.client = client;
   }

   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      final char c = e.getKeyChar();
      if ('h' == c)
      {
         ClientState.print_help_message();
         return;
      }
      else if ('d' == c)
      {
         client.modelArray.get(client.currentModel).debug = ! client.modelArray.get(client.currentModel).debug;
       //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
      }
      else if ('c' == c)
      {
         Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
         System.out.print("Clipping is turned ");
         System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
      }
      else if ('/' == c)
      {
         client.currentModel = (client.currentModel + 1) % client.modelArray.size();
         client.scene.setModel( 0, client.modelArray.get(client.currentModel) );
      }
      else if ('?' == c)
      {
         client.currentModel = (client.currentModel - 1);
         if (client.currentModel < 0) client.currentModel = client.modelArray.size() - 1;
         client.scene.setModel( 0, client.modelArray.get(client.currentModel) );
      }
      else if ('p' == c)
      {
         client.scene.camera.perspective = ! client.scene.camera.perspective;
         final String p = client.scene.camera.perspective ? "perspective" : "orthographic";
         System.out.println("Using " + p + " projection");
      }
      else if ('m' == c) //display transformation information
      {
         client.displayTransformations = ! client.displayTransformations;
      }
      else if ('s' == c) // Scale the current model 10% smaller.
      {
         // This does NOT work with perspective projection.
         scaleModel(client.currentModel, 1.0/1.1);
      }
      else if ('S' == c) // Scale the current model 10% larger.
      {
         // This does NOT work with perspective projection.
         scaleModel(client.currentModel, 1.1);
      }
      else if ('x' == c) // Translate the current model.
      {
         moveModel(client.currentModel, -0.1, 0, 0); // left
      }
      else if ('X' == c)
      {
         moveModel(client.currentModel, +0.1, 0, 0); // right
      }
      else if ('y' == c)
      {
         moveModel(client.currentModel, 0, -0.1, 0); // down
      }
      else if ('Y' == c)
      {
         moveModel(client.currentModel, 0, +0.1, 0); // up
      }
      else if ('z' == c)
      {
         moveModel(client.currentModel, 0, 0, -0.1); // back
      }
      else if ('Z' == c)
      {
         moveModel(client.currentModel, 0, 0, +0.1); // forward
      }

      if (client.displayTransformations &&
            ('m'==c||'/'==c||'?'==c
           ||'s'==c||'x'==c||'y'==c||'z'==c
           ||'S'==c||'X'==c||'Y'==c||'Z'==c))
      {
         System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                           "deltaZ = %.2f, deltaS = %.2f\n",
                           client.deltaX[client.currentModel],
                           client.deltaY[client.currentModel],
                           client.deltaZ[client.currentModel],
                           client.deltaS[client.currentModel]);
      }

      // Render again.
      final FrameBuffer fb = client.fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(client.scene, fb);
      client.fbp.update();
   }


   private void scaleModel(int currentModel, double deltaS)
   {
      client.deltaS[currentModel] *= deltaS;
      final Model m = client.scene.getModel(0);
      for (int i = 0; i < m.vertexList.size(); ++i)
      {
         final Vertex v = m.vertexList.get(i);
         m.vertexList.set(i, new Vertex(v.x * deltaS,
                                        v.y * deltaS,
                                        v.z * deltaS));
      }
   }


   private void moveModel(int currentModel, double deltaX, double deltaY, double deltaZ)
   {
      client.deltaX[currentModel] += deltaX;
      client.deltaY[currentModel] += deltaY;
      client.deltaZ[currentModel] += deltaZ;
      final Model m = client.scene.getModel(0);
      for (int i = 0; i < m.vertexList.size(); ++i)
      {
         final Vertex v = m.vertexList.get(i);
         m.vertexList.set(i, new Vertex(v.x + deltaX,
                                        v.y + deltaY,
                                        v.z + deltaZ));
      }
   }
}
