/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

/**
   This keyboard handler translates ALL the models, not just the current model.
*/
class KeyHandler_v2 extends KeyAdapter
{
   private final ClientState client;

   public KeyHandler_v2(ClientState client)
   {
      this.client = client;
   }

   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      final char c = e.getKeyChar();
      if ('h' == c)
      {
         ClientState.print_help_message();
         return;
      }
      else if ('d' == c)
      {
         client.modelArray.get(client.currentModel).debug = ! client.modelArray.get(client.currentModel).debug;
       //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
      }
      else if ('c' == c)
      {
         Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
         System.out.print("Clipping is turned ");
         System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
      }
      else if ('/' == c)
      {
         client.currentModel = (client.currentModel + 1) % client.modelArray.size();
         client.scene.setModel( 0, client.modelArray.get(client.currentModel) );
      }
      else if ('?' == c)
      {
         client.currentModel = (client.currentModel - 1);
         if (client.currentModel < 0) client.currentModel = client.modelArray.size() - 1;
         client.scene.setModel( 0, client.modelArray.get(client.currentModel) );
      }
      else if ('p' == c)
      {
         client.scene.camera.perspective = ! client.scene.camera.perspective;
         final String p = client.scene.camera.perspective ? "perspective" : "orthographic";
         System.out.println("Using " + p + " projection");
      }
      else if ('m' == c) //display transformation information
      {
         client.displayTransformations = ! client.displayTransformations;
      }
      else if ('s' == c) // Scale ALL the models 10% smaller.
      {
         // This does NOT work with perspective projection.
         scaleModels(1.0/1.1);
      }
      else if ('S' == c) // Scale ALL the models 10% larger.
      {
         // This does NOT work with perspective projection.
         scaleModels(1.1);
      }
      else if ('x' == c) // Translate ALL the models.
      {
         moveModels(-0.1, 0, 0); // left
      }
      else if ('X' == c)
      {
         moveModels(+0.1, 0, 0); // right
      }
      else if ('y' == c)
      {
         moveModels(0, -0.1, 0); // down
      }
      else if ('Y' == c)
      {
         moveModels(0, +0.1, 0); // up
      }
      else if ('z' == c)
      {
         moveModels(0, 0, -0.1); // back
      }
      else if ('Z' == c)
      {
         moveModels(0, 0, +0.1); // forward
      }

      if (client.displayTransformations && ('m'==c
           ||'s'==c||'x'==c||'y'==c||'z'==c
           ||'S'==c||'X'==c||'Y'==c||'Z'==c))
      {
         System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                           "deltaZ = %.2f, deltaS = %.2f\n",
                           client.deltaX[0],
                           client.deltaY[0],
                           client.deltaZ[0],
                           client.deltaS[0]);
      }

      // Render again.
      final FrameBuffer fb = client.fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(client.scene, fb);
      client.fbp.update();
   }


   private void scaleModels(double deltaS)
   {
      client.deltaS[0] *= deltaS;
      for (final Model m : client.modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {  // This does NOT work with perspective projection.
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x * deltaS,
                                           v.y * deltaS,
                                           v.z * deltaS));
         }
      }
   }


   private void moveModels(double deltaX, double deltaY, double deltaZ)
   {
      client.deltaX[0] += deltaX;
      client.deltaY[0] += deltaY;
      client.deltaZ[0] += deltaZ;
      for (final Model m : client.modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x + deltaX,
                                           v.y + deltaY,
                                           v.z + deltaZ));
         }
      }
   }
}
