/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.io.File;

/**

*/
public class InteractiveModels_R2 extends InteractiveAbstractClient_R2
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveModels_R2()
   {
      // Create the Scene object that we shall render
      scene = new Scene();

      // Create several Model objects.
      modelArray.add( new ObjSimpleModel(new File("../assets/apple.obj")) );
      modelArray.add( new ObjSimpleModel(new File("../assets/cow.obj")) );
      modelArray.add( new ObjSimpleModel(new File("../assets/galleon.obj")) );
      modelArray.add( new ObjSimpleModel(new File("../assets/teapot.obj")) );
      modelArray.add( new ObjSimpleModel(new File("../assets/cessna.obj")) );
      modelArray.add( new Sphere(1.0, 30, 30) );
      modelArray.add( new Cylinder(0.5, 1.0, 20, 20) );
      modelArray.add( new Torus(0.75, 0.25, 25, 25) );
      modelArray.add( new Cube2(15, 15, 15) );
      modelArray.add( new ObjSimpleModel(new File("../assets/small_rhombicosidodecahedron.obj")) );
      final Model floor = new PanelXZ(-2, 2, -2, 0);  // floor
      final Model wall  = new PanelXY(-4, 4, -1, 3);  // wall
      final Model airplane = new ObjSimpleModel(new File("../assets/cessna.obj"));

      // Add four models to the Scene.
      scene.addModel(modelArray.get(1), // cow
                     floor,
                     wall,
                     airplane);
      currentModel = 1;

      deltaX = new double[modelArray.size()];
      deltaY = new double[modelArray.size()];
      deltaZ = new double[modelArray.size()];
      deltaS = new double[modelArray.size()];
      java.util.Arrays.fill(deltaS, 1.0);

      // Position the wall, floor and airplane.
      for (int i = 0; i < wall.vertexList.size(); ++i)
      {
         final Vertex v = wall.vertexList.get(i);
         wall.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 4));
      }
      for (int i = 0; i < floor.vertexList.size(); ++i)
      {
         final Vertex v = floor.vertexList.get(i);
         floor.vertexList.set(i, new Vertex(v.x,
                                            v.y - 1,
                                            v.z - 2));
      }
      for (int i = 0; i < airplane.vertexList.size(); ++i)
      {
         final Vertex v = airplane.vertexList.get(i);
         airplane.vertexList.set(i, new Vertex(v.x + 2.5,
                                               v.y,
                                               v.z - 2));
      }
      // Push the models away from where the camera is.
      for (final Model m : modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 2));
         }
      }


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Register this object as the event listener for JFrame events.
      jf.addKeyListener(this);
      jf.addComponentListener(this);
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveModels_R2()
      );
   }
}
