/*

*/

import models_pc.*;
import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.io.File;
import java.util.List;
import java.util.ArrayList;

/**

*/
public class InteractivePointCloudGRSModels_R2
{
   private final List<Model> modelArray = new ArrayList<>();
   private int currentModel = 0;
   private double deltaX = 0.0;
   private double deltaY = 0.0;
   private double deltaS = 1.0;
   private boolean displayTransformations = false;

   /**
      This constructor first instantiates the Scene object
      and initializes it with appropriate geometry. Then
      this constructor instantiates the GUI and its event
      handlers.
   */
   public InteractivePointCloudGRSModels_R2()
   {
      // Create the Scene object that we shall render
      final Scene scene = new Scene();
      scene.camera.perspective = false;

      // Instantiate all the grs models.
      final int n = 10;
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/bronto.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/rex.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/bronto_v2.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/rex_v2.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/dragon.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/vinci.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/usa.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/usa_v2.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/birdhead.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/knight.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/house.grs"), n));
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/scene.grs"), n));

      // Create a set of x and y axes.
      final Model axes = new Axes2D(-1, +1, -1, +1, 20, 20);

      // Add two models to the Scene.
      scene.addModel(modelArray.get(0),
                     axes);

      Rasterize_Clip.doClipping = true;

      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      final FrameBufferPanel fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create a KeyListener event handler object that will respond to
      // keyboard events and redraw the Scene into the FrameBufferPanel.
      jf.addKeyListener( new KeyListener()
      {
         // Implement the KeyListener interface.
         @Override public void keyPressed(KeyEvent e){}
         @Override public void keyReleased(KeyEvent e){}
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );

            final char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            else if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
            }
            else if ('c' == c)
            {
               Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
               System.out.print("Clipping is turned ");
               System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
            }
            else if ('/' == c)
            {
               currentModel = (currentModel + 1) % modelArray.size();
               scene.setModel( 0, modelArray.get(currentModel) );
            }
            else if ('?' == c)
            {
               currentModel = (currentModel - 1);
               if (currentModel < 0) currentModel = modelArray.size() - 1;
               scene.setModel( 0, modelArray.get(currentModel) );
            }
            else if ('m' == c) //display transformation information
            {
               displayTransformations = ! displayTransformations;
            }
            else if ('s' == c) // Scale all the models 10% smaller.
            {
               deltaS /= 1.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     // This does NOT work with perspective projection.
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x / 1.1,
                                                    v.y / 1.1,
                                                    v.z / 1.1));
                  }
            }
            else if ('S' == c) // Scale all the models 10% larger.
            {
               deltaS *= 1.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     // This does NOT work with perspective projection.
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x * 1.1,
                                                    v.y * 1.1,
                                                    v.z * 1.1));
                  }
            }
            else if ('x' == c)
            {
               deltaX -= 0.01;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x - 0.01,
                                                    v.y,
                                                    v.z));
                  }
            }
            else if ('X' == c)
            {
               deltaX += 0.01;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x + 0.01,
                                                    v.y,
                                                    v.z));
                  }
            }
            else if ('y' == c)
            {
               deltaY -= 0.01;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x,
                                                    v.y - 0.01,
                                                    v.z));
                  }
            }
            else if ('Y' == c)
            {
               deltaY += 0.01;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x,
                                                    v.y + 0.01,
                                                    v.z));
                  }
            }

            if (displayTransformations && ('m'==c
                 ||'s'==c||'x'==c||'y'==c
                 ||'S'==c||'X'==c||'Y'==c))
            {
               System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                                 "deltaS = %.2f\n",
                                  deltaX,        deltaY,
                                  deltaS);
            }

            // Render again.
            final FrameBuffer fb = fbp.getFrameBuffer();
            fb.clearFB();
            Pipeline.render(scene, fb);
            fbp.update();
         }
      });

      // Create a ComponentListener event handler object that will respond
      // to windows events and redraw the scene into the FrameBufferPanel.
      jf.addComponentListener( new ComponentListener()
      {
         // Implement the ComponentListener interface.
         @Override public void componentMoved(ComponentEvent e){}
         @Override public void componentHidden(ComponentEvent e){}
         @Override public void componentShown(ComponentEvent e){}
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );

            //System.out.printf("JFrame [w = %d, h = %d]: " +
            //                  "FrameBufferPanel [w = %d, h = %d].\n",
            //                  jf.getWidth(), jf.getHeight(),
            //                  fbp.getWidth(), fbp.getHeight());

            // Get the new size of the FrameBufferPanel.
            final int w = fbp.getWidth();
            final int h = fbp.getHeight();

            // Create a new FrameBuffer that fits the FrameBufferPanel.
            final FrameBuffer fb = new FrameBuffer(w, h);
            fbp.setFrameBuffer(fb);
            Pipeline.render(scene, fb);
            fbp.update();
         }
      });
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> {new InteractivePointCloudGRSModels_R2();}
      );
   }


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the '/' key to cycle through the models.");
      System.out.println("Use the x/X, y/Y, keys to translate the models along the x and y axes.");
      System.out.println("Use the s/S keys to scale the size of the models.");
      System.out.println("Use the 'm' key to toggle the display of transformation information.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
