/*

*/

import models_pc.*;
import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.io.File;
import java.util.List;
import java.util.ArrayList;

/**

*/
public class InteractivePointCloudsAll_R2
{
   final private List<Model> modelArray = new ArrayList<>();
   private int currentModel = 0;
   private double deltaX = 0.0;
   private double deltaY = 0.0;
   private double deltaZ = 0.0;
   private double deltaS = 1.0;
   private boolean displayTransformations = false;

   /**
      This constructor first instantiates the Scene object
      and initializes it with appropriate geometry. Then
      this constructor instantiates the GUI and its event
      handlers.
   */
   public InteractivePointCloudsAll_R2()
   {
      // Create the Scene object that we shall render
      final Scene scene = new Scene();

      final int n = 100;  // size of each point cloud
      modelArray.add(new PointCloud_Square(1.0, n));
      modelArray.add(new PointCloud_SquareSolid(1.0, n));
      modelArray.add(new PointCloud_Circle(1.0, n));
      modelArray.add(new PointCloud_Disk(1.0, n, n));
      modelArray.add(new PointCloud_Disk(1.0, n, 10));
      modelArray.add(new PointCloud_Disk(1.0, 10, n));
      modelArray.add(new PointCloud_Disk(1.0, 3*n, n/2));
      modelArray.add(new PointCloud_DiskSector(1.0, Math.PI/2, 3*Math.PI/2, n/2, n));
      //modelArray.add(new PointCloud_Ring(1.0, 0.25, 3, 16));
      //modelArray.add(new PointCloud_RingSector(1.0, 0.25, Math.PI/2, 3*Math.PI/2, 3, 8));
      modelArray.add(new PointCloud_Triangle(45, 6)); // try 7
      // cubes
      modelArray.add(new PointCloud_Box(1.0, 1.0, 1.0, n));
      modelArray.add(new PointCloud_Cube(n));
      modelArray.add(new PointCloud_CubeSolid(n));
      // polyhedra
      modelArray.add(new PointCloud_Tetrahedron(n));
      modelArray.add(new PointCloud_TetrahedronSolid(5));
      modelArray.add(new PointCloud_Tetrahedron(true, n));
      modelArray.add(new PointCloud_Octahedron(n));
      modelArray.add(new PointCloud_Icosahedron());
      modelArray.add(new PointCloud_Dodecahedron());
      // cones
      modelArray.add(new PointCloud_Cone(1.0, 1.0, n, n));
      modelArray.add(new PointCloud_ConeSector(1.0, 1.0, 0.5, 0, 2*Math.PI, n/2, n/4));
      modelArray.add(new PointCloud_ConeSector(1.0, 1.0, 0.5, Math.PI/2, 3*Math.PI/2, n, n));
      modelArray.add(new PointCloud_ConeFrustum(1.0, 0.5, 0.5, n, n));
      modelArray.add(new PointCloud_ConeFrustum(0.5, 1.0, 0.5, n, n));
      // cylinders
      modelArray.add(new PointCloud_Cylinder(0.5, 1.0, n, n));
      modelArray.add(new PointCloud_CylinderSector(0.5, 1.0, Math.PI/2, 3*Math.PI/2, n, n));
      // spheres
      modelArray.add(new PointCloud_Sphere(1.0, n, n));
      modelArray.add(new PointCloud_Torus(0.75, 0.25, n, n));
      // model files
      modelArray.add(new PointCloud_GRSModel(new File("../assets/grs/bronto.grs"), n));
      modelArray.add(new PointCloud_ObjSimpleModel(new File("../assets/cow.obj"), n));
      modelArray.add(new PointCloud_ObjSimpleModelSolid(new File("../assets/cow.obj"), 2));
      // parametric curves and surfaces
      modelArray.add(new PointCloud_ParametricCurve());
      modelArray.add(new PointCloud_ParametricSurface());
      modelArray.add(new PointCloud_ParametricSurface(
                (s,t)->s*Math.cos(t*Math.PI),
                (s,t)->t,
                (s,t)->s*Math.sin(t*Math.PI),
                -1, 1, -1, 1, n, n));
      modelArray.add(new PointCloud_ParametricSurface(
                (u,v)->0.3*(1-u)*(3+Math.cos(v))*Math.sin(4*Math.PI*u),
                (u,v)->0.3*(3*u+(1-u)*Math.sin(v)),
                (u,v)->0.3*(1-u)*(3+Math.cos(v))*Math.cos(4*Math.PI*u),
                0, 1, 0, 2*Math.PI, n, n));
      modelArray.add(new PointCloud_SurfaceOfRevolution());
      modelArray.add(new PointCloud_SurfaceOfRevolution(t->0.5*(1+t*t), -1, 1, n, n));
      modelArray.add(new PointCloud_SurfaceOfRevolution(t->t, t->4*t*(1-t), 0, 1, n, n));

      // Add a model to the Scene.
      scene.addModel(modelArray.get(currentModel));

      // Push the models away from where the camera is.
      for (final Model m : modelArray)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 2));
         }
      }

      Rasterize_Clip.doClipping = true;


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      final FrameBufferPanel fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create a KeyListener event handler object that will respond to
      // keyboard events and redraw the Scene into the FrameBufferPanel.
      jf.addKeyListener( new KeyListener()
      {
         // Implement the KeyListener interface.
         @Override public void keyPressed(KeyEvent e){}
         @Override public void keyReleased(KeyEvent e){}
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );

            final char c = e.getKeyChar();
            if ('h' == c)
            {
               print_help_message();
               return;
            }
            else if ('d' == c)
            {
               modelArray.get(currentModel).debug = ! modelArray.get(currentModel).debug;
             //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
            }
            else if ('c' == c)
            {
               Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
               System.out.print("Clipping is turned ");
               System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
            }
            else if ('/' == c)
            {
               currentModel = (currentModel + 1) % modelArray.size();
               scene.setModel( 0, modelArray.get(currentModel) );
            }
            else if ('?' == c)
            {
               currentModel = (currentModel - 1);
               if (currentModel < 0) currentModel = modelArray.size() - 1;
               scene.setModel( 0, modelArray.get(currentModel) );
            }
            else if ('p' == c)
            {
               scene.camera.perspective = ! scene.camera.perspective;
               final String p = scene.camera.perspective ? "perspective" : "orthographic";
               System.out.println("Using " + p + " projection");
            }
            else if ('m' == c) //display transformation information
            {
               displayTransformations = ! displayTransformations;
            }
            else if ('s' == c) // Scale all the models 10% smaller.
            {
               deltaS /= 1.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     // This does NOT work with perspective projection.
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x / 1.1,
                                                    v.y / 1.1,
                                                    v.z / 1.1));
                  }
            }
            else if ('S' == c) // Scale all the models 10% larger.
            {
               deltaS *= 1.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     // This does NOT work with perspective projection.
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x * 1.1,
                                                    v.y * 1.1,
                                                    v.z * 1.1));
                  }
            }
            else if ('x' == c)
            {
               deltaX -= 0.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x - 0.01,
                                                    v.y,
                                                    v.z));
                  }
            }
            else if ('X' == c)
            {
               deltaX += 0.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x + 0.01,
                                                    v.y,
                                                    v.z));
                  }
            }
            else if ('y' == c)
            {
               deltaY -= 0.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x,
                                                    v.y - 0.01,
                                                    v.z));
                  }
            }
            else if ('Y' == c)
            {
               deltaY += 0.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x,
                                                    v.y + 0.01,
                                                    v.z));
                  }
            }
            else if ('z' == c)
            {
               deltaZ -= 0.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x,
                                                    v.y,
                                                    v.z - 0.1));
                  }
            }
            else if ('Z' == c)
            {
               deltaZ += 0.1;
               for (final Model m : modelArray)
                  for (int i = 0; i < m.vertexList.size(); ++i)
                  {
                     final Vertex v = m.vertexList.get(i);
                     m.vertexList.set(i, new Vertex(v.x,
                                                    v.y,
                                                    v.z + 0.1));
                  }
            }

            if (displayTransformations && ('m'==c
                 ||'s'==c||'x'==c||'y'==c||'z'==c
                 ||'S'==c||'X'==c||'Y'==c||'Z'==c))
            {
               System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                                 "deltaZ = %.2f, deltaS = %.2f\n",
                                  deltaX,        deltaY,
                                  deltaZ,        deltaS);
            }

            // Render again.
            final FrameBuffer fb = fbp.getFrameBuffer();
            fb.clearFB();
            Pipeline.render(scene, fb);
            fbp.update();
         }
      });

      // Create a ComponentListener event handler object that will respond
      // to windows events and redraw the scene into the FrameBufferPanel.
      jf.addComponentListener( new ComponentListener()
      {
         // Implement the ComponentListener interface.
         @Override public void componentMoved(ComponentEvent e){}
         @Override public void componentHidden(ComponentEvent e){}
         @Override public void componentShown(ComponentEvent e){}
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );

            //System.out.printf("JFrame [w = %d, h = %d]: " +
            //                  "FrameBufferPanel [w = %d, h = %d].\n",
            //                  jf.getWidth(), jf.getHeight(),
            //                  fbp.getWidth(), fbp.getHeight());

            // Get the new size of the FrameBufferPanel.
            final int w = fbp.getWidth();
            final int h = fbp.getHeight();

            // Create a new FrameBuffer that fits the FrameBufferPanel.
            final FrameBuffer fb = new FrameBuffer(w, h);
            fbp.setFrameBuffer(fb);
            Pipeline.render(scene, fb);
            fbp.update();
         }
      });
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> {new InteractivePointCloudsAll_R2();}
      );
   }


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off for the current model.");
      System.out.println("Use the '/' key to cycle through the models.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the models along the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the models.");
      System.out.println("Use the 'm' key to toggle the display of transformation information.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
