/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a circle in the xy-plane centered at the origin.
*/
public class PointCloud_Circle extends Model
{
   /**
      Create a circle in the xy-plane with radius 1 and
      with 100 points around the circumference.
   */
   public PointCloud_Circle( )
   {
      this(1, 100);
   }


   /**
      Create a circle in the xy-plane with radius {@code r}
      and with 100 points around the circumference.

      @param r  radius of the circle
   */
   public PointCloud_Circle(final double r)
   {
      this(r, 100);
   }


   /**
      Create a circle in the xy-plane with radius {@code r}
      and with {@code n} points around the circumference.

      @param r  radius of the circle
      @param n  number of points in the circle's circumference
   */
   public PointCloud_Circle(final double r, final int n)
   {
      super("Point Cloud Circle (n="+n+")");

      // Create the circle's geometry.
      final double deltaTheta = (2.0 * Math.PI) / n;

      // Vertices.
      for (int i = 0; i < n; ++i)
      {
         final double c = Math.cos(i * deltaTheta);
         final double s = Math.sin(i * deltaTheta);
         addVertex( new Vertex(r * c, r * s, 0) );
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); i++)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Circle
