/*

*/

package models_pc;

/**
   Create a point cloud model of a right circular cone with its base
   parallel to the xz-plane and its apex on the positive y-axis.

   @see ConeFrustum
*/
public class PointCloud_Cone extends PointCloud_ConeSector
{
   /**
      Create a right circular cone with its base in the xz-plane,
      a base radius of 1, height 1, and apex on the positve y-axis.
   */
   public PointCloud_Cone( )
   {
      this(1, 1, 100, 100);
   }


   /**
      Create a right circular cone with its base in the xz-plane,
      a base radius of {@code r}, height {@code h}, and apex on
      the y-axis.

      @param r  radius of the base in the xz-plane
      @param h  height of the apex on the y-axis
   */
   public PointCloud_Cone(final double r, final double h)
   {
      this(r, h, 100, 100);
   }


   /**
      Create a right circular cone with its base in the xz-plane,
      a base radius of {@code r}, height {@code h}, and apex on
      the y-axis.
   <p>
      The last two parameters determine the number of lines of
      longitude and the number of circles of latitude in the model.

      @param r  radius of the base in the xz-plane
      @param h  height of the apex on the y-axis
      @param n  number of circles of latitude around the cone
      @param k  number of lines of longitude
   */
   public PointCloud_Cone(final double r, final double h,
                          final int n, final int k)
   {
      super(r, h, h, 0, 2*Math.PI, n, k);
      name = "Point Cloud Cone";
   }
}//PointCloud_Cone
