/*

*/

package models_pc;

/**
   Create a point cloud  model of a right circular cylinder
   with its axis along the y-axis.

   @see PointCloudCylinderSector
*/
public class PointCloud_Cylinder extends PointCloud_CylinderSector
{
   /**
      Create a right circular cylinder with radius 1 and its
      axis along the y-axis from {@code y = -1} to {@code y = 1}.
   */
   public PointCloud_Cylinder( )
   {
      this(1, 1, 15, 16);
   }


   /**
      Create a right circular cylinder with radius {@code r} and
      its axis along the y-axis from {@code y = -h} to {@code y = h}.

      @param r  radius of the cylinder
      @param h  height of the cylinder (from -h to h along the y-axis)
   */
   public PointCloud_Cylinder(final double r, final double h)
   {
      this(r, h, 15, 16);
   }


   /**
      Create a right circular cylinder with radius {@code r} and
      its axis along the y-axis from {@code y = -h} to {@code y = h}.
   <p>
      The last two parameters determine the number of lines of longitude
      and the number of circles of latitude in the model.

      @param r  radius of the cylinder
      @param h  height of the cylinder (from -h to h along the y-axis)
      @param n  number of circles of latitude around the cylinder
      @param k  number of lines of longitude
   */
   public PointCloud_Cylinder(final double r, final double h, final int n, final int k)
   {
      super(r, h, 0, 2*Math.PI, n, k);
      name = "Point Cloud Cylinder";
   }
}//PointCloud_Cylinder
