/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a disk
   in the xy-plane centered at the origin.
*/
public class PointCloud_Disk extends Model
{
   /**
      Create a disk in the xy-plane with radius 1 and
      with 100 points around the circumference and
      along each spoke.
   */
   public PointCloud_Disk( )
   {
      this(1, 100, 100);
   }


   /**
      Create a disk in the xy-plane with radius {@code r}
      and with 100 points around the circumference and
      along each spoke.

      @param r  radius of the disk
   */
   public PointCloud_Disk(final double r)
   {
      this(r, 100, 100);
   }


   /**
      Create a disk in the xy-plane with radius {@code r}
      and with {@code n} points around the circumference
      and with {@code k} points along each spoke.

      @param r  radius of the disk
      @param n  number of points in the disk's circumference
   */
   public PointCloud_Disk(final double r, final int n)
   {
      this(r, n, n);
   }


   /**
      Create a disk in the xy-plane with radius {@code r}
      and with {@code n} points around the circumference.

      @param r  radius of the disk
      @param n  number of points in the disk's circumference
      @param k  number of points in the disk's spokes
   */
   public PointCloud_Disk(final double r, final int n, final int k)
   {
      super("Point Cloud Disk");

      // Create the disk's geometry.
      final double deltaTheta = (2.0 * Math.PI) / n;
      final double deltaR = r / k;

      // Vertices.
      for (int j = 0; j < k; ++j)
      {
         final double r2 = r - (j * deltaR);
         for (int i = 0; i < n; ++i)
         {
            final double c = Math.cos(i * deltaTheta);
            final double s = Math.sin(i * deltaTheta);
            addVertex( new Vertex(r2 * c, r2 * s, 0) );
         }
      }
      addVertex(new Vertex(0,0,0)); // center

      // Line segments.
      for (int i = 0; i < vertexList.size(); i++)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Disk
