/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a sphere centered at the origin.
*/
public class PointCloud_Sphere extends Model
{
   /**
      Create a sphere of radius 1 centered at the origin
      and with 100 points around each circle of latitude
      and longitude.
   */
   public PointCloud_Sphere()
   {
      this(1.0, 100, 100);
   }


   /**
      Create a sphere of radius {@code r} centered at the origin
      and with 100 points around each circle of latitude
      and longitude.

      @param r  radius of the sphere
   */
   public PointCloud_Sphere(final double r)
   {
      this(r, 100, 100);
   }


   /**
      Create a sphere of radius {@code r} centered at the origin
      and with {@code n} points around each circle of latitude
      and longitude.

      @param r  radius of the sphere
      @param n  number of circles of latitude
      @param k  number of half circles of longitude
   */
   public PointCloud_Sphere(final double r, final int n, final int k)
   {
      super("Point Cloud Sphere (n="+n+")");

      // Create the geometry for the Model.
      final double deltaPhi = Math.PI / (n + 1);
      final double deltaTheta = (2 * Math.PI) / k;
      // Vertices.
      for (int j = 0; j < k; ++j) // choose an angle of longitude
      {
         final double c1 = Math.cos(j * deltaTheta);
         final double s1 = Math.sin(j * deltaTheta);
         for (int i = 0; i < n; ++i) // choose an angle of latitude
         {
            final double c2 = Math.cos(deltaPhi + i * deltaPhi);
            final double s2 = Math.sin(deltaPhi + i * deltaPhi);
            addVertex( new Vertex( r * s2 * c1,
                                   r * c2,
                                  -r * s2 * s1 ) );
         }
      }
      addVertex( new Vertex(0,  r, 0) );  // north pole
      addVertex( new Vertex(0, -r, 0) );  // south pole

      // Line segments.
      for (int i = 0; i < vertexList.size(); i++)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Sphere
