/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a torus.
*/
public class PointCloud_Torus extends Model
{
   /**
      Create a torus with a circle of revolution with radius 3/4
      and a cross section circle (circle of longitude)
      with radius 1/4.
   */
   public PointCloud_Torus( )
   {
      this(0.75, 0.25, 100, 100);
   }


   /**
      Create a torus with a circle of revolution with radius {@code r1}
      and a cross section circle (circle of longitude) with radius
      {@code r2}.

      @param r1  radius of the circle of revolution
      @param r2  radius of the cross section circle (circle of longitude)
   */
   public PointCloud_Torus(final double r1, final double r2)
   {
      this(r1, r2, 100, 100);
   }


   /**
      Create a torus with a circle of revolution with radius {@code r1}
      and a cross section circle (circle of longitude) with radius
      {@code r2}.
   <p>
      The last two parameters determine the number of circles of
      longitude and the number of circles of latitude in the model
      and how many points are on each circle of latitude or langitude.

      @param r1  radius of the circle of revolution
      @param r2  radius of the cross section circle (circle of longitude)
      @param n   number of points on a circle of latitude
      @param k   number of points on a circle of longitude
   */
   public PointCloud_Torus(final double r1, final double r2,
                           final int n, final int k)
   {
      super("Point Cloud Torus (n="+n+")");

      // Create the torus's geometry.
      final double deltaPhi = (2 * Math.PI) / n;
      final double deltaTheta = (2 * Math.PI) / k;

      // Vertices.
      for (int j = 0; j < k; ++j) // choose a rotation around the y-axis
      {
         final double c1 = Math.cos(j * deltaTheta);
         final double s1 = Math.sin(j * deltaTheta);
         for (int i = 0; i < n; ++i)  // go around a cross section circle
         {
            final double c2 = Math.cos(i * deltaPhi);
            final double s2 = Math.sin(i * deltaPhi);
            addVertex(new Vertex( (r1 + r2*s2) * c1,
                                        r2*c2,
                                 -(r1 + r2*s2) * s1 ));
         }
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Torus
